/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cono.h"

Cono::Cono( QWidget *parent, const char *name )
        : QGLWidget( parent, name )
//  : QGLWidget( QGLFormat( SingleBuffer|NoDepthBuffer ), parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;
	n = 8;
	initCone( C1, 8, 0.4, 0.6, 0.0, 0.0, 0.0 ); 
}


void Cono::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cono::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cono::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cono::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
	
	emit changeSize ( );

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
}

QSizePolicy Cono::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cono::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	glRotatef ( angle , 0.0, 0.0, 1.0);

	drawCone( C1, 8 );
	
	glPopMatrix( );
	swapBuffers( );
}

void Cono::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
	glShadeModel (GL_SMOOTH );
}

void Cono::initCone( float c[][3], int i, float r, float a, 
	float cx, float cy, float cz )
{
	int j;

	c[0][0] = cx;
	c[0][1] = cy + a;
	c[0][2] = cz;

	float theta = 2*3.1415923/i;
	for ( j=1; j<=i; j++ ) {
		float ang = (j-1)*theta;
		c[j][0] = cx + r*cos(ang);	
		c[j][1] = cy;	
		c[j][2] = cz + r*sin(ang);	
	}
}

void Cono::drawCone( float c[][3], int i )
{
	glBegin( GL_TRIANGLES );
	for ( int j=1; j<=i; j++ ){
		glColor3f( 1.0, 1.0, 1.0 );
		int k = j%i + 1;
		glVertex3fv( &c[k][0] );
		glVertex3fv( &c[j][0]);
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3fv( &c[0][0] );
	}
	glEnd( );
}

